searchNodes=[{"doc":"A module for working with finite-state machines and defining finite-state machine callbacks. Unlike :gen_statem , which implements the machine as an Erlang process, this state machine can be implemented with a distributed storage mechanism such as a database. Example The following example shows a simple pushbutton model for a toggling pushbutton. You can push the button and it replies if it went on or off, and you can ask for a count of how many times it has been pushed to switch on. The following is the complete callback module file push_button.ex : defmodule PushButton do @behaviour Mimzy import Ecto.Query , only : [ from : 2 ] @impl Mimzy def handle_event ( :create ) do { 1 , [ %{ id : id } ] } = Repo . insert_all ( &quot;buttons&quot; , [ [ count : 0 , state : &quot;off&quot; ] ] , returning : [ :id ] ) id end @spec handle_event ( Mimzy . id ( ) , Mimzy . event ( ) ) :: term def handle_event ( id , event ) , do : Repo . transaction ( fn -&gt; Mimzy . handle_event ( id , event , __MODULE__ ) end ) @impl Mimzy def init ( id ) do case Repo . one ( from b in button_query ( id ) , lock : &quot;FOR UPDATE&quot; , select : { b . state , b . count } ) do { state , count } -&gt; { :cont , state , count } nil -&gt; { :halt , :error } end end @impl Mimzy def handle_event ( &quot;off&quot; , :push , id , count ) do id |&gt; button_query ( ) |&gt; Repo . update_all ( set : [ count : count + 1 , state : &quot;on&quot; ] ) :on end def handle_event ( &quot;on&quot; , :push , id , count ) do id |&gt; button_query ( ) |&gt; Repo . update_all ( set : [ count : count , state : &quot;off&quot; ] ) :off end def handle_event ( _state , { :put_count , new_count } , id , _count ) do id |&gt; button_query ( ) |&gt; Repo . update_all ( set : [ count : new_count ] ) :ok end def handle_event ( _state , :delete , id , _count ) do { 1 , nil } = id |&gt; button_query ( ) |&gt; Repo . delete_all ( ) :ok end def handle_event ( _state , :get_count , _id , count ) , do : count @spec button_query ( Mimzy . id ( ) ) :: Ecto.Query . t ( ) defp button_query ( id ) do from b in &quot;buttons&quot; , where : b . id == ^ id end end Usage would be: id = PushButton . handle_event ( :create ) #=&gt; 123 PushButton . handle_event ( id , :get_count ) #=&gt; 0 PushButton . handle_event ( id , :push ) #=&gt; :on PushButton . handle_event ( id , :get_count ) #=&gt; 1 PushButton . handle_event ( id , :push ) #=&gt; :off PushButton . handle_event ( id , { :put_count , 99 } ) #=&gt; :ok PushButton . handle_event ( id , :get_count ) #=&gt; 99 PushButton . handle_event ( id , :delete ) #=&gt; :ok PushButton . handle_event ( id , :push ) #=&gt; :error","ref":"Mimzy.html","title":"Mimzy","type":"behaviour"},{"doc":"Called when a state machine event is ready to be handled and the machine is in a nonexistent or pseudo state. This is the function that would create a new finite-state machine.","ref":"Mimzy.html#c:handle_event/1","title":"Mimzy.handle_event/1","type":"callback"},{"doc":"Handles a state machine event. This function delegates to the callbacks implemented in the given module .","ref":"Mimzy.html#handle_event/3","title":"Mimzy.handle_event/3","type":"function"},{"doc":"This function is called after init/1 . This is the function that would transition a finite-state machine from one state to another.","ref":"Mimzy.html#c:handle_event/4","title":"Mimzy.handle_event/4","type":"callback"},{"doc":"Called when a state machine event is ready to be handled. This function is called before handle_event/4 in order to retrieve the state and any machine-associated data. The return value is expected to be {:cont, state, data} to continue the event handling {:halt, term} to halt the event handling and return the term","ref":"Mimzy.html#c:init/1","title":"Mimzy.init/1","type":"callback"},{"doc":"","ref":"Mimzy.html#t:data/0","title":"Mimzy.data/0","type":"type"},{"doc":"","ref":"Mimzy.html#t:event/0","title":"Mimzy.event/0","type":"type"},{"doc":"","ref":"Mimzy.html#t:id/0","title":"Mimzy.id/0","type":"type"},{"doc":"","ref":"Mimzy.html#t:state/0","title":"Mimzy.state/0","type":"type"},{"doc":"This is a new major release of Mimzy that improves the API.","ref":"changelog.html","title":"Changelog for v2.x","type":"extras"},{"doc":"Enhancements Update documentation","ref":"changelog.html#v2-0-1-2023-03-13","title":"Changelog for v2.x - v2.0.1 (2023-03-13)","type":"extras"},{"doc":"Backwards incompatible changes id and event parameters have been swapped in handle_event/3 function Enhancements Make handle_event/1 callback optional Add documentation for handle_event/3 function","ref":"changelog.html#v2-0-0-2023-03-13","title":"Changelog for v2.x - v2.0.0 (2023-03-13)","type":"extras"},{"doc":"","ref":"readme.html","title":"Mimzy","type":"extras"},{"doc":"Add :mimzy to your list of dependencies in mix.exs : defp deps do [ { :mimzy , &quot;~&gt; 2.0&quot; } ] end","ref":"readme.html#installation","title":"Mimzy - Installation","type":"extras"},{"doc":"Mimzy is a finite-state machine library for Elixir. Here is an example: defmodule ButtonMachine do @behaviour Mimzy def handle_event ( :create ) , do : _id = ButtonRepo . insert ( count : 0 , state : :off ) def handle_event ( id , event ) , do : Mimzy . handle_event ( id , event , __MODULE__ ) def init ( id ) do button = ButtonRepo . get_and_lock ( id ) { :cont , button . state , button . count } end def handle_event ( :off , :push , id , count ) do :ok = ButtonRepo . update_and_unlock ( id , count : count + 1 , state : :on ) :on end def handle_event ( :on , :push , id , _count ) do :ok = ButtonRepo . update_and_unlock ( id , state : :off ) :off end def handle_event ( _state , :get_count , id , count ) do :ok = ButtonRepo . unlock ( id ) count end end","ref":"readme.html#about","title":"Mimzy - About","type":"extras"},{"doc":"This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/ .","ref":"readme.html#license","title":"Mimzy - License","type":"extras"}]